/*
 *
 */

#include <avkon.hrh>
#include <aknnotewrappers.h>
#include <apgtask.h>
#include <eikspane.h>
#include <stringloader.h>
#include <carlock.rsg>
#include <f32file.h>
#include <s32file.h>
#include <commdbconnpref.h>
#include <sys/reent.h>

#include "carlock.hrh"
#include "carlock_appui.h"
#include "carlock_appview.h"
#include "carlock_console.h"
#include "carlock_settings.h"
#include "carlock_engine.h"

void CCarLockAppUi::ConstructL()
{
    // Initialise app UI with standard value.
    BaseConstructL(CAknAppUi::EAknEnableSkin);

    CEikStatusPane* sp = StatusPane();
    iNaviPane  = (CAknNavigationControlContainer*)sp->ControlL(TUid::Uid(EEikStatusPaneUidNavi));
    iDecorator = iNaviPane->ResourceDecorator();
    iTabGroup  = (CAknTabGroup*)iDecorator->DecoratedControl();

    // Create view object
    iAppView = CCarLockAppView::NewL( ClientRect() );
    iAppView->SetDevice( _L("---") );
    iAppView->SetStat( _L("unknown") );
    iAppView->SetVoltage( 0.0 );

    // Create console object
    iConsole = CCarLockConsole::NewL( ClientRect() );
    iConsole->MakeVisible( EFalse );

    // Create settings object
    iSettings = new (ELeave) CCarLockSettings;
    iSettings->SetMopParent( this );
    iSettings->ConstructL();
    iSettings->MakeVisible( EFalse );
    iSettings->SetRect( ClientRect() );
    iSettings->ActivateL();

    // Create stream object
    iEngine = CCarLockEngine::NewL( this );

    {
	const TDesC& device  = iSettings->GetDevice();
	const TDesC& address = iSettings->GetAddress();
	TInt         channel = iSettings->GetChannel();

	iEngine->Connect( device, address, channel );
    }
}

// -----------------------------------------------------------------------------
// CCarLockAppUi::CCarLockAppUi()
// C++ default constructor can NOT contain any code, that might leave.
// -----------------------------------------------------------------------------
//
CCarLockAppUi::CCarLockAppUi() { }

// -----------------------------------------------------------------------------
// CCarLockAppUi::~CCarLockAppUi()
// Destructor.
// -----------------------------------------------------------------------------
//
CCarLockAppUi::~CCarLockAppUi()
{
    if( iEngine )   { iEngine->Disconnect( EFalse ); delete iEngine; iEngine = NULL; }
    if( iAppView )  { delete iAppView;  iAppView = NULL;  }
    if( iConsole )  { delete iConsole;  iConsole = NULL;  }
    if( iSettings ) { delete iSettings;	iSettings = NULL; }
}

// -----------------------------------------------------------------------------
// CCarLockAppUi::HandleCommandL()
// Takes care of command handling.
// -----------------------------------------------------------------------------
//
void CCarLockAppUi::HandleCommandL( TInt aCommand )
{
    switch( aCommand )
    {
        case EEikCmdExit:
        case EAknSoftkeyExit:
	    iSettings->SaveSettings();
	    CloseSTDLIB();
            Exit();
            break;

	case ECarLockCommandConnect:
	    {
		const TDesC& device  = iSettings->GetDevice();
		const TDesC& address = iSettings->GetAddress();
		TInt         channel = iSettings->GetChannel();

		iEngine->Connect( device, address, channel );
	    }
            break;

	case ECarLockCommandSettings:
	    SelectView( ECarLockSettingsTab );
	    break;

	case ECarLockCommandOpen:
	    iEngine->Open();
	    break;

	case ECarLockCommandClose:
	    iEngine->Close();
	    break;

	case ECarLockCommandHide:
	    Hide();
	    break;

	default:
	    break;
    }
}

// -----------------------------------------------------------------------------
//  Called by the framework when the application status pane
//  size is changed.  Passes the new client rectangle to the
//  AppView
// -----------------------------------------------------------------------------
//
void CCarLockAppUi::HandleStatusPaneSizeChange()
{
	iAppView->SetRect( ClientRect() );
} 

void CCarLockAppUi::HandleWsEventL(const TWsEvent& aEvent, CCoeControl* aDestination)
{
    if( iTabGroup == NULL ) goto fall_to_std;

    TInt active = iTabGroup->ActiveTabId();

    switch( aEvent.Type() )
    {
	case EEventKeyDown :

	    TInt count  = iTabGroup->TabCount();

	    switch( aEvent.Key()->iScanCode )
	    {
		case EStdKeyLeftArrow:
        	    if ( active <= 1 ) return;
            	    active--;
		    SelectView(active);
        	    return;

    		case EStdKeyRightArrow:
        	    if ( active >= count ) return;
            	    active++;
		    SelectView(active);
        	    return;
	    }

	    if( active == ECarLockSettingsTab ) break;

	    switch( aEvent.Key()->iScanCode )
	    {
		case 18:
		    if( iEngine ) iEngine->Stat();
		    break;
		case EStdKeyYes:
		    if( iEngine )
		    {
			switch( iEngine->DevState() )
			{
			    case EDevOpened:
				iEngine->Close();
				break;
			    case EDevClosed:
				iEngine->Open();
				break;
			    case EDevUnknown:
				break;
			}
		    }
		    return;
	    }

	    break;

	case EEventKeyUp :

	    if( active == ECarLockSettingsTab ) goto fall_to_std;
	    break;

	case KAknUidValueEndKeyCloseEvent:
	    return;

	default :
	    break;
    }

fall_to_std:
    // fall to standard handler
    CAknAppUi::HandleWsEventL( aEvent, aDestination );
}

void CCarLockAppUi::Hide()
{
    TApaTask task( iEikonEnv->WsSession() );
    task.SetWgId( CEikonEnv::Static()->RootWin().Identifier() );
    task.SendToBackground();
}

void CCarLockAppUi::SelectView( TInt id )
{
    if( iTabGroup && iAppView && iSettings )
    {
	iTabGroup->SetActiveTabById( id );
	switch( id )
	{
	    case ECarLockMainTab:
		iAppView->MakeVisible( ETrue );
		iConsole->MakeVisible( EFalse );
		iSettings->MakeVisible( EFalse );
		RemoveFromStack( iSettings );
		break;

	    case ECarLockConsoleTab:
		iAppView->MakeVisible( EFalse );
		iConsole->MakeVisible( ETrue );
		iSettings->MakeVisible( EFalse );
		RemoveFromStack( iSettings );
		break;

	    case ECarLockSettingsTab:
		iAppView->MakeVisible( EFalse );
		iConsole->MakeVisible( EFalse );
		iSettings->MakeVisible( ETrue );
		AddToStackL( iSettings );
		break;
	}
    }
}
