/*
 *
 */

#include <coemain.h>
#include <eikenv.h>
#include <bitstd.h>
#include <bitdev.h>

#include "carlock_appview.h"

CCarLockAppView* CCarLockAppView::NewL( const TRect& aRect )
{
    CCarLockAppView* self = CCarLockAppView::NewLC( aRect );
    CleanupStack::Pop( self );
    return self;
}

CCarLockAppView* CCarLockAppView::NewLC( const TRect& aRect )
{
    CCarLockAppView* self = new ( ELeave ) CCarLockAppView;
    CleanupStack::PushL( self );

    self->ConstructL( aRect );

    return self;
}

CCarLockAppView::CCarLockAppView() 
{ 
    iLedColor = KRgbBlack;
}

void CCarLockAppView::ConstructL( const TRect& aRect )
{
    iTimer = CCarLockTimer::NewL( *this );
    CreateWindowL();
    SetRect( aRect );
    ActivateL();
}

void CCarLockAppView::CreateBackBufferL()
{
    iBmp = new (ELeave) CFbsBitmap;
    User::LeaveIfError( iBmp->Create( Size(), iEikonEnv->DefaultDisplayMode() ) );
    iDev = CFbsBitmapDevice::NewL( iBmp );
    User::LeaveIfError( iDev->CreateContext( iGc ) );

    iGcSize = iBmp->SizeInPixels();

    TRect rect( TPoint( 0, 0 ), iGcSize );
    iGc->SetBrushColor( KRgbBlack );
    iGc->SetBrushStyle( CGraphicsContext::ESolidBrush );
    iGc->Clear( rect );
}

void CCarLockAppView::ReleaseBackBuffer()
{
    if (iGc)  { delete iGc;  iGc  = NULL; }
    if (iDev) { delete iDev; iDev = NULL; }
    if (iBmp) { delete iBmp; iBmp = NULL; }
    iGcSize = TSize(0, 0);
}

CCarLockAppView::~CCarLockAppView() 
{
    ReleaseBackBuffer();
    delete iTimer;
}

void CCarLockAppView::SizeChanged()
{  
    ReleaseBackBuffer();
    CreateBackBufferL();
    DrawToBackBuffer();
}

/* Time-out timer callback */
void CCarLockAppView::TimerExpired()
{
    iLedColor = KRgbBlack;
    DrawLed( 3, iLedColor, ETrue );
}

void CCarLockAppView::Draw( const TRect& aRect ) const
{
    SystemGc().BitBlt( aRect.iTl, iBmp, aRect );
}

#define BOX_Y(x)	( h * (x) + 1 )

void CCarLockAppView::DrawToBackBuffer()
{
    DrawStatic( 0, iDevice, EFalse );
    DrawStatic( 1, iStat, EFalse );
    DrawStatic( 2, iVoltage, EFalse );
    DrawLed( 3, iLedColor, EFalse );
}

void CCarLockAppView::DrawStatic( TInt y, const TDesC& aText, TBool aDrawNow )
{
    if( !iGc ) return;

    const CFont *font = CEikonEnv::Static()->TitleFont();
    TInt h = font->HeightInPixels() + 6;
    TInt baseline = h / 2 + font->AscentInPixels() / 2;
    TRect rect( TPoint( 0, BOX_Y(y) ), TSize( iGcSize.iWidth, h ) );

    
    iGc->SetBrushColor( KRgbBlack );
    iGc->SetBrushStyle( CGraphicsContext::ESolidBrush );
    iGc->Clear( rect );

    iGc->SetBrushStyle( CGraphicsContext::ENullBrush );
    iGc->SetPenColor( KRgbGreen );
    iGc->DrawRoundRect( rect,  TSize( 5, 5 ) );

    iGc->UseFont( font );
    iGc->SetPenColor( KRgbWhite );
    iGc->DrawText( aText, rect,  baseline, CGraphicsContext::ECenter, 0 );
    iGc->DiscardFont();
    
    if( aDrawNow ) DrawNow( rect );
}

void CCarLockAppView::DrawLed( TInt y, const TRgb& aColor, TBool aDrawNow )
{
    if( !iGc ) return;

    const CFont *font = CEikonEnv::Static()->TitleFont();
    TInt h = font->HeightInPixels() + 6;
    TRect rect( TPoint( 0, BOX_Y(y) ), TSize( h, h ) );

    
    iGc->SetBrushColor( KRgbBlack );
    iGc->SetBrushStyle( CGraphicsContext::ESolidBrush );
    iGc->Clear( rect );

    iGc->SetBrushStyle( CGraphicsContext::ENullBrush );
    iGc->SetPenColor( KRgbGreen );
    iGc->DrawRoundRect( rect,  TSize( 5, 5 ) );

    iGc->SetBrushColor( aColor );
    iGc->SetBrushStyle( CGraphicsContext::ESolidBrush );
    iGc->SetPenColor( KRgbBlack );
    TRect el( rect );
    el.Shrink( 1, 1 );
    iGc->DrawEllipse( el );

    if( aDrawNow ) DrawNow( rect );
}

void CCarLockAppView::SetDevice( const TDesC& aText )
{
    iDevice.Copy( aText );
    DrawStatic( 0, iDevice, ETrue );
}

void CCarLockAppView::SetStat( const TDesC& aText  )
{
    iStat.Format( _L("Lock: %S"), &aText );
    DrawStatic( 1, iStat, ETrue );
}

void CCarLockAppView::SetVoltage( const TReal32 aValue )
{
    iVoltage.Format( _L("Voltage: %.1f V"), aValue );
    DrawStatic( 2, iVoltage, ETrue );
}

void CCarLockAppView::FlashLed()
{
    iTimer->Cancel();
    iLedColor = KRgbRed;
    DrawLed( 3, iLedColor, ETrue );
    iTimer->After( 250000 );
}
