/*
 *
 */

#include <coemain.h>
#include <eikenv.h>
#include <bitstd.h>
#include <bitdev.h>

#include "carlock_console.h"

CCarLockConsole* CCarLockConsole::NewL( const TRect& aRect )
{
    CCarLockConsole* self = CCarLockConsole::NewLC( aRect );
    CleanupStack::Pop( self );
    return self;
}

CCarLockConsole* CCarLockConsole::NewLC( const TRect& aRect )
{
    CCarLockConsole* self = new ( ELeave ) CCarLockConsole;
    CleanupStack::PushL( self );

    self->ConstructL( aRect );

    return self;
}

CCarLockConsole::CCarLockConsole() { }

void CCarLockConsole::ConstructL( const TRect& aRect )
{
    CreateWindowL();
    SetRect( aRect );
    ActivateL();
}

void CCarLockConsole::CreateBackBufferL()
{
    iBmp = new (ELeave) CFbsBitmap;
    User::LeaveIfError( iBmp->Create( Size(), iEikonEnv->DefaultDisplayMode() ) );
    iDev = CFbsBitmapDevice::NewL( iBmp );
    User::LeaveIfError( iDev->CreateContext( iGc ) );

    iGcSize = iBmp->SizeInPixels();

    TRect rect( TPoint( 0, 0 ), iGcSize );
    iGc->SetBrushColor( KRgbBlack );
    iGc->SetBrushStyle( CGraphicsContext::ESolidBrush );
    iGc->Clear( rect );
}

void CCarLockConsole::ReleaseBackBuffer()
{
    if (iGc)  { delete iGc;  iGc  = NULL; }
    if (iDev) { delete iDev; iDev = NULL; }
    if (iBmp) { delete iBmp; iBmp = NULL; }
    iGcSize = TSize(0, 0);
}

CCarLockConsole::~CCarLockConsole() 
{
    ReleaseBackBuffer();
}

void CCarLockConsole::SizeChanged()
{  
    ReleaseBackBuffer();
    CreateBackBufferL();
}

void CCarLockConsole::Draw( const TRect& aRect ) const
{
    SystemGc().BitBlt( aRect.iTl, iBmp, aRect );
}

void CCarLockConsole::AddLine(const TDesC& aText, TBool aDrawNow )
{
    if( !iGc ) return;

    const CFont *font = CEikonEnv::Static()->DenseFont();
    TInt sh = font->HeightInPixels() + 4;
    TInt baseline = sh / 2 + font->AscentInPixels() / 2;
    TInt yoff = ( ( iGcSize.iHeight - 1 ) % sh ) + sh + 1;
    TRect rstr( TPoint( 0, iGcSize.iHeight - sh ), TSize( iGcSize.iWidth, sh ) );
    TRect rscr( TPoint( 0, yoff ), TSize( iGcSize.iWidth, iGcSize.iHeight - yoff ) );

    /* scroll contents of console */
    iGc->CopyRect( TPoint( 0, -sh ), rscr );
    /* clear last string */
    iGc->SetBrushColor( KRgbBlack );
    iGc->SetBrushStyle( CGraphicsContext::ESolidBrush );
    iGc->Clear( rstr );
    /* print last string */
    iGc->UseFont( font );
    iGc->SetBrushStyle( CGraphicsContext::ENullBrush );
    iGc->SetPenColor( KRgbWhite );
    iGc->DrawText( aText, rstr,  baseline, CGraphicsContext::ELeft, 0 );
    iGc->DiscardFont();

    if( aDrawNow )
    {
	TRect rect( TPoint( 0, 1 ), TSize( iGcSize.iWidth, iGcSize.iHeight - 1 ) );
	DrawNow( rect );
    }
}

void CCarLockConsole::Print( TRefByValue<const TDesC> aFmt, ... )
{
    VA_LIST list;
    VA_START( list, aFmt );
    iText.Zero();
    iText.AppendFormatList( aFmt, list );
    VA_END(list);
    AddLine( iText, ETrue );
}
