/*
 *
 */

#ifndef __CARLOCK_ENGINE_H__
#define __CARLOCK_ENGINE_H__

#include <es_sock.h>
#include <in_sock.h>
#include <bt_sock.h>
#include <mdaaudiotoneplayer.h>

#include "carlock_timer.h"

class CRx;
class CTx;
class CCarLockAppUi;

const TInt KCarLockDNSTimeOut = 20000000; // 20 seconds time-out

enum TCarLockState 
{
    EDisconnected,	/* disconnected state */
    EResolving,		/* resolving device name */
    EConnecting,	/* connecting to device */
    EConnected,		/* connected to device */
    ESending		/* sending data to device */
};

enum TCarLockDevState 
{
    EDevUnknown,	/* Unknown state */
    EDevOpened,		/* Lock open */
    EDevClosed		/* Lock closed */
};

enum TCarLockCmdSent
{
    ECmdNone,		/* nothing sent */
    ECmdStat,		/* status command sent */
    ECmdOpen,		/* OPEN command sent */
    ECmdClose		/* CLOSE command sent */
};

class CCarLockEngine : public CActive,
		       public MCarLockTimeOutNotify,
		       public MMdaAudioToneObserver
{
    public:

        static CCarLockEngine* NewL( CCarLockAppUi* parent );
        virtual ~CCarLockEngine();

        TInt Connect( const TDesC& aDevice, const TDesC& aAddress, TInt aChannel );
        void Disconnect( TBool aRetry );

	TCarLockState State();
	TCarLockDevState DevState();

	void Open();
	void Close();
	void Stat();

    protected:

	void DoCancel();
	void RunL();
	void TimerExpired();
	void MatoPrepareComplete(TInt aError);
        void MatoPlayComplete(TInt aError);

    private:    // Functions from base classes

        CCarLockEngine( CCarLockAppUi* parent );
	void ConstructL();

	void SocketConnect( TBTDevAddr& aDevAddr, TInt aChannel );

	void SetState( TCarLockState aState );
	void HandleReply( TDes8& aReply );

	void Bin2Hex( TDes8& dst, const TDesC8& src );
	void SendPass( const TDesC& pass, const TDesC8& challenge );
	void Send( TRefByValue<const TDesC8> aFmt, ... );
	void RequestData();

    private:

	TCarLockState		iState;

	CCarLockAppUi*		iParent;
	CCarLockTimer*		iTimer;
	CMdaAudioToneUtility* 	iToneUtility;

	RSocketServ		iSocketServer;
	RHostResolver		iResolver;
	RSocket			iSocket;

	TBuf<256>		iDevice;
	TBuf<64>		iAddress;
	TBTDevAddr		iDevAddr;
	TInt			iChannel;

	TInquirySockAddr	iSockAddr;
	TNameEntry		iNameEntry;

	TInt			iMaxReconn;
	TInt			iReconn;

	TBuf8<256>		iMessage;
	TBuf<512>		iUnicode;

	TBuf8<4096>		iBuffer;
	TBuf8<4096>		iReply;

        TSockXfrLength		iSentLen;
        TSockXfrLength		iRcvdLen;

	TCarLockDevState	iDevState;
	TCarLockCmdSent		iCmdSent;
};

#endif // __CARLOCK_ENGINE_H_
