/*
 * Settings page
 */

#include <carlock.rsg>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <utf.h>
#include "carlock.hrh"
#include "carlock_settings.h"

#define CONFIG_FILE "C:\\Data\\carlock.conf"

void CCarLockSettings::ConstructL()
{ 
    /* set defaults */
    device.Copy(_L("BT Lock"));
    address.Copy(_L(""));
    channel.Copy(_L("1"));
    delay.Copy(_L("1000"));
    searchint.Copy(_L("5000"));
    pass.Copy(_L("12345"));
    vddscale.Copy(_L("10.0"));

    /* try to read values from config file */
    ParseConfig( CONFIG_FILE );

    /* create settings page */
    CAknSettingItemList::ConstructFromResourceL( R_CARLOCK_SETTING_LIST );
}

void CCarLockSettings::SizeChanged()
{
	// if size changes, make sure component takes whole available space
	CEikFormattedCellListBox *listbox = ListBox();
	if ( listbox ) 
        {
	    listbox->SetRect( Rect() );
        }
}

CAknSettingItem * CCarLockSettings::CreateSettingItemL( TInt id )
{
    CAknSettingItem *settingItem = NULL;

    switch( id )
    {
	case ECarLockSettingDevice:
	    settingItem = new (ELeave) CAknTextSettingItem( id, device );
	    break;
	case ECarLockSettingAddress:
	    settingItem = new (ELeave) CAknTextSettingItem( id, address );
	    break;
	case ECarLockSettingChannel:
	    settingItem = new (ELeave) CAknTextSettingItem( id, channel );
	    break;
	case ECarLockSettingDelay:
	    settingItem = new (ELeave) CAknTextSettingItem( id, delay );
	    break;
	case ECarLockSettingSearchInt:
	    settingItem = new (ELeave) CAknTextSettingItem( id, searchint );
	    break;
	case ECarLockSettingVddScale:
	    settingItem = new (ELeave) CAknTextSettingItem( id, vddscale );
	    break;
	case ECarLockSettingPassword:
	    settingItem = new (ELeave) CAknPasswordSettingItem( id, CAknPasswordSettingItem::EAlpha, pass );
	    break;

	default:
	    break;
    }
        
    return settingItem;
}

void CCarLockSettings::TDesC_to_str( char* dst, const TDesC& src )
{
    TInt	len = src.Size();
    TPtr8	ptr( (TUint8*)dst, len + 1 );

    CnvUtfConverter::ConvertFromUnicodeToUtf8( ptr, src );
    ptr.ZeroTerminate();
}

const TDesC& CCarLockSettings::GetDevice()
{
    StoreSettingsL();
    return device;
}

const TDesC& CCarLockSettings::GetAddress()
{
    StoreSettingsL();
    return address;
}

TInt CCarLockSettings::GetDelay()
{
    StoreSettingsL();
    TInt ret;
    TLex iLex(delay);

    iLex.Val(ret);
    return ret;
}

TReal32 CCarLockSettings::GetVddScale()
{
    StoreSettingsL();
    TReal32 ret;
    TLex iLex(vddscale);

    iLex.Val(ret);
    return ret;
}

TInt CCarLockSettings::GetSearchInt()
{
    StoreSettingsL();
    TInt ret;
    TLex iLex(searchint);

    iLex.Val(ret);
    return ret;
}

const TDesC& CCarLockSettings::GetPassword()
{
    StoreSettingsL();
    return pass;
}

TInt CCarLockSettings::GetChannel()
{
    StoreSettingsL();
    TInt ret;
    TLex iLex(channel);

    iLex.Val(ret);
    return ret;
}

void CCarLockSettings::GetConfigInt( char * string, char * name, TInt& val )
{
    char *p = strstr( string, name );

    if( !p ) return;

    p += strlen( name );
    while( *p && isspace( *p ) ) p++;
    if( *p && *p != '\n' && *p != '\r' )
    {
        val = strtol(p, NULL, 0);
    }
}

void CCarLockSettings::GetConfigStr( char * string, char * name, TDes& val )
{
    char *p = strstr( string, name );

    if( !p ) return;

    p += strlen( name );
    while( *p && isspace( *p ) ) p++;
    if( *p && *p != '\n' && *p != '\r' )
    {
	char *s = p;
	while( *p && *p != '\n' && *p != '\r' ) p++; *p = 0;
	TPtrC8 ptr( (const TUint8 *)s, strlen( s ) );
	CnvUtfConverter::ConvertToUnicodeFromUtf8( val, ptr );
    }
}

void CCarLockSettings::ParseConfig( char * config )
{
    FILE *f;
    char str[256];

    f = fopen( config, "r" );

    if( !f ) return;

    while( fgets( str, sizeof(str), f ) )
    {
    	if( str[0] == '\n' || str[0] == '\r' || str[0] == '#' ) continue;
	GetConfigStr( str, "device",    device    );
	GetConfigStr( str, "address",   address   );
    	GetConfigStr( str, "channel",   channel   );
    	GetConfigStr( str, "delay",     delay     );
    	GetConfigStr( str, "searchint", searchint );
    	GetConfigStr( str, "pass",      pass      );
    	GetConfigStr( str, "vddscale",  vddscale  );
    }
    fclose( f );
}

void CCarLockSettings::SaveConfig( char * config )
{
    FILE *f;
    char str[256];

    f = fopen( config, "w" );

    if( !f ) return;

    TDesC_to_str( str, device );
    fprintf( f, "device %s\n", str );
    TDesC_to_str( str, address );
    fprintf( f, "address %s\n", str );
    TDesC_to_str( str, channel );
    fprintf( f, "channel %s\n", str );
    TDesC_to_str( str, delay );
    fprintf( f, "delay %s\n", str );
    TDesC_to_str( str, searchint );
    fprintf( f, "searchint %s\n", str );
    TDesC_to_str( str, pass );
    fprintf( f, "pass %s\n", str );
    TDesC_to_str( str, vddscale );
    fprintf( f, "vddscale %s\n", str );
    fclose( f );
}

void CCarLockSettings::SaveSettings()
{
    StoreSettingsL();
    SaveConfig( CONFIG_FILE );
}
